Index: chan_mgcp.c
===================================================================
RCS file: /usr/cvsroot/asterisk/channels/chan_mgcp.c,v
retrieving revision 1.79.2.4
diff -u -5 -r1.79.2.4 chan_mgcp.c
--- chan_mgcp.c	17 Oct 2004 04:57:10 -0000	1.79.2.4
+++ chan_mgcp.c	25 Oct 2004 14:01:24 -0000
@@ -1076,22 +1076,26 @@
 
 static struct ast_frame *mgcp_rtp_read(struct mgcp_subchannel *sub)
 {
 	/* Retrieve audio/etc from channel.  Assumes sub->lock is already held. */
 	struct ast_frame *f;
+	static struct ast_frame null_frame = { AST_FRAME_NULL, };
 	f = ast_rtp_read(sub->rtp);
+	/* Don't send RFC2833 if we're not supposed to */
+	if (f && (f->frametype == AST_FRAME_DTMF) && !(sub->parent->dtmfmode & MGCP_DTMF_RFC2833))
+		return &null_frame;
 	if (sub->owner) {
 		/* We already hold the channel lock */
 		if (f->frametype == AST_FRAME_VOICE) {
 			if (f->subclass != sub->owner->nativeformats) {
 				ast_log(LOG_DEBUG, "Oooh, format changed to %d\n", f->subclass);
 				sub->owner->nativeformats = f->subclass;
 				ast_set_read_format(sub->owner, sub->owner->readformat);
 				ast_set_write_format(sub->owner, sub->owner->writeformat);
 			}
             /* Courtesy fearnor aka alex@pilosoft.com */
-            if (sub->parent->dtmfmode & MGCP_DTMF_INBAND) {
+            if ((sub->parent->dtmfmode & MGCP_DTMF_INBAND) && (sub->parent->dsp)) {
 #if 0
                 ast_log(LOG_NOTICE, "MGCP ast_dsp_process\n");
 #endif
                 f = ast_dsp_process(sub->owner, sub->parent->dsp, f);
             }
@@ -1101,22 +1105,16 @@
 }
 
 
 static struct ast_frame  *mgcp_read(struct ast_channel *ast)
 {
-	struct ast_frame *fr;
+	struct ast_frame *f;
 	struct mgcp_subchannel *sub = ast->pvt->pvt;
-	static struct ast_frame null_frame = { AST_FRAME_NULL, };
 	ast_mutex_lock(&sub->lock);
-	fr = mgcp_rtp_read(sub);
-	if (!(sub->parent->dtmfmode & MGCP_DTMF_RFC2833)) {
-		if (fr && (fr->frametype == AST_FRAME_DTMF)) {
-			fr = &null_frame;
-		}
-	}
+	f = mgcp_rtp_read(sub);
 	ast_mutex_unlock(&sub->lock);
-	return fr;
+	return f;
 }
 
 static int mgcp_write(struct ast_channel *ast, struct ast_frame *frame)
 {
 	struct mgcp_subchannel *sub = ast->pvt->pvt;
